BeginPackage["HEPMath`LHAPDF`",
  {"HEPMath`Tensors`", "HEPMath`Lorentz`", "HEPMath`PaVe`"}]


LHAPDFMembers::usage = "LHAPDFMembers[name] returns the number of members
in the PDF set with name name."

LHAPDFOpen::usage = "LHAPDFOpen[name, n] opens member n of the PDF set with
name name and returns the LHAPDF ID of that member."

LHAPDFOpenID::usage = "LHAPDFOpenID[id] opens the PDF set with the LHAPDF ID
id."

LHAPDFClose::usage = "LHAPDFClose[id] closes the PDF set with
LHAPDF ID id (if it was previously opened with LHAPDFOpen)."

LHAPDFFlavors::usage = "LHAPDFFlavors[id] returns the list of flavors (i.e. PDG
numbers of partons) present in the PDF set with LHAPDF ID id. The set must
previously have been opened with LHAPDFOpen."

LHAPDF::usage = "LHAPDF[id, pid, x, Q] returns the value of the PDF
 (x*f(x)) of the parton with PDG number pid in the set with LHAPDF ID id for
momentum fraction x and factorization scale Q. The set id must previously have
been opened with LHAPDFOpen."

LHAPDFAlphaS::usage = "LHAPDFAlphaS[id, Q] returns the value of the strong
coupling alpha_s for the pdf set with LHAPDF ID id and factorization scale
Q. The set id must previously have been opened with LHAPDFOpen."


Begin["`Protected`"]

lhapdfMembers::usage = "lhapdfMembers[name] returns the number of members
in the PDF set with name name."

lhapdfOpen::usage = "lhapdfOpen[name, n] opens member n of the PDF set with
name name and returns the LHAPDF ID of that member."

lhapdfOpenID::usage = "lhapdfOpenID[id] opens the PDF set with the LHAPDF ID
id."

lhapdfClose::usage = "lhapdfClose[id] closes the PDF set with
LHAPDF ID id (if it was previously opened with lhapdfOpen)."

lhapdfFlavors::usage = "lhapdfFlavors[id] returns the list of flavors (i.e. PDG
numbers of partons) present in the PDF set with LHAPDF ID id. The set must
previously have been opened with lhapdfOpen."

lhapdfGet::usage = "lhapdfGet[id, pid, x, Q] returns the value of the PDF
 (x*f(x)) of the parton with PDG number pid in the set with LHAPDF ID id for
momentum fraction x and factorization scale Q. The set id must previously have
been opened with lhapdfOpen."

lhapdfAlphaS::usage = "lhapdfAlphaS[id, Q] returns the value of the strong
coupling alpha_s for the pdf set with LHAPDF ID id and factorization scale
Q. The set id must previously have been opened with lhapdfOpen."

End[]


Begin["`Private`"]

Needs["HEPMath`Config`"]

AppendTo[$ContextPath, "HEPMath`LHAPDF`Protected`"]

SetAttributes[libSet, HoldFirst]

libSet[var_, func_, args_, res_] := If[$LHAPDFLib =!= "",
  var = LibraryFunctionLoad[$LHAPDFLib, func, args, res],
  var := (Message[$LHAPDFLib::noif]; Abort[])
]

libSet[lhapdfMembers, "numMembers", {"UTF8String"}, Integer]

LHAPDFMembers[s_String] := lhapdfMembers[s]

libSet[lhapdfOpen, "openPDF", {"UTF8String", Integer}, Integer]

LHAPDFOpen[s_String, mem_Integer] := lhapdfOpen[s, mem]

libSet[lhapdfOpenID, "openID", {Integer}, Integer]

LHAPDFOpenID[id_Integer] := lhapdfOpenID[id]

libSet[lhapdfClose, "closePDF", {Integer}, Integer]

LHAPDFClose[id_Integer] := lhapdfClose[id]

libSet[lhapdfFlavors, "getFlavors", {Integer}, {Integer, 1}]

LHAPDFFlavors[id_Integer] := lhapdfFlavors[id]

libSet[lhapdfGet, "getPDF", {Integer, Integer, Real, Real}, Real]

LHAPDF[id_Integer, pid_Integer,
       x_ /; Element[x, Reals] === True,
       q_ /; Element[q, Reals] === True] := lhapdfGet[id, pid, N[x], N[q]]

libSet[lhapdfAlphaS, "getAlphaS", {Integer, Real}, Real]

LHAPDFAlphaS[id_Integer, q_  /; Element[q, Reals] === True] :=
  lhapdfAlphaS[id, N[q]]

$ContextPath = DeleteCases[$ContextPath, "HEPMath`LHAPDF`Protected`"]


End[]

EndPackage[]

